############################################################################
# Copyright 2015 Skyscanner Ltd                                            #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License");          #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#    http://www.apache.org/licenses/LICENSE-2.0                            #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################
import unittest

from hamcrest import equal_to, assert_that, raises, calling
from selenium.common.exceptions import TimeoutException

from pages.element_with_traits import ElementWithTraits
from pages.exceptions import IllegalStateException


class ElementWithTraitsTest(unittest.TestCase):
    """
    Unit test for ElementWithTraits class.
    """

    def test_can_wait_until_traits_are_loaded(self):
        element = ElementWithTraits('an_element').add_trait(lambda: True, 'always present')
        assert_that(element.wait_until_loaded(5, 1), equal_to(element))

    def test_raises_timeout_when_traits_do_not_load(self):
        element = ElementWithTraits('an_element').add_trait(lambda: False, 'never loading')
        assert_that(calling(element.wait_until_loaded).with_args(1, 0.5), raises(TimeoutException))

    def test_raises_exception_when_there_are_no_traits(self):
        element = ElementWithTraits('an_element')
        assert_that(calling(element.wait_until_loaded).with_args(1, 0.5),
                    raises(IllegalStateException))



############################################################################
# Copyright 2015 Skyscanner Ltd                                            #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License");          #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#    http://www.apache.org/licenses/LICENSE-2.0                            #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################
from pages.ui_component import UIComponent


class RadioButton(UIComponent):

    def select(self):
        return self.locate().click()



############################################################################
# Copyright 2015 Skyscanner Ltd                                            #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License");          #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#    http://www.apache.org/licenses/LICENSE-2.0                            #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################
import unittest

from hamcrest import equal_to, assert_that, is_not, raises
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.common.by import By
from selenium.webdriver.remote.command import Command
from selenium.webdriver.remote.webelement import WebElement

from pages.ui_component import UIComponent
from test.utils.mocks import MockedWebElement, MockedWebDriver


class UiComponentTest(unittest.TestCase):
    """
    Unit test for UiComponent class.
    """

    def __init__(self, methodName='runTest'):
        super(UiComponentTest, self).__init__(methodName)
        self.driver = MockedWebDriver()

    def test_component_can_be_created_from_web_element(self):
        component = UIComponent(self.driver, 'a component', [By.ID, 'theId'])
        a_web_element = WebElement(None, '')
        ##
        component.from_web_element(a_web_element)
        ##
        assert_that(component.locate(), equal_to(a_web_element),
                    "fromWebElement should store a reference to passed WebElement")

    def test_raise_error_when_trying_to_construct_from_wrong_type(self):
        def construct_from_foo():
            driver = MockedWebDriver()
            return UIComponent(driver, 'a component').from_web_element(Foo())
        assert_that(construct_from_foo, raises(TypeError), "passing unexpected type should causes exception")

    def test_web_element_is_stored_with_caching(self):
        component = UIComponent(self.driver, 'a component', [By.ID, 'theid'])
        self.driver.set_dom_element([By.ID, 'theid'])
        element_first_time = component.cache().locate()
        ##
        self.driver.reset_dom_elements()
        self.driver.set_dom_element([By.ID, 'theid'])
        element_second_time = component.locate()
        ##
        assert_that(element_first_time, equal_to(element_second_time),
                    "when cached web_element() should return always the same WebElement object")

    def test_web_element_is_evaluated_every_time_without_caching(self):
        component = UIComponent(self.driver, 'a component', [By.ID, 'theid'])
        self.driver.set_dom_element([By.ID, 'theid'])
        element_first_time = component.locate()
        ##
        self.driver.reset_dom_elements()
        self.driver.set_dom_element([By.ID, 'theid'])
        element_second_time = component.locate()
        ##
        assert_that(element_first_time, is_not(equal_to(element_second_time)),
                    "when cached web_element() should return always the same WebElement object")

    def test_component_is_found_when_has_all_traits(self):
        component = UIComponent(self.driver, 'a_component', [By.ID, 'an_id'])
        ##
        component.add_trait(lambda: True, 'always true')
        ##
        assert_that(component.is_found(), equal_to(True), "component should be found when all traits are present")

    def test_component_is_not_found_when_trait_raises_an_exception(self):
        component = UIComponent(self.driver, 'a_component', [By.ID, 'an_id'])
        ##
        component.add_trait(raise_exception, 'always true')
        ##
        assert_that(component.is_found(), equal_to(False),
                    "component should not be found when evaluating traits throws an exception")

    def test_is_present(self):
        self.driver.set_dom_element([By.ID, 'an_id'])
        component = UIComponent(self.driver, 'a_component', [By.ID, 'an_id'])

        component.is_present()

        assert_that(self.driver.has_fulfilled_expectations(), equal_to(True),
                    "component should be able to locate element it is scope")

    def test_component_can_be_clicked(self):
        self.driver.set_dom_element([By.ID, 'an_id'])

        self.driver.set_expected_command(Command.CLICK_ELEMENT, {'sessionId': self.driver.session_id,
                                         'id': self.driver.get_id_for_stored_element([By.ID, 'an_id'])})

        component = UIComponent(self.driver, 'a_component', [By.ID, 'an_id'])
        ##
        component.click()
        ##
        assert_that(self.driver.has_fulfilled_expectations(), equal_to(True),
                    "clicking on component should result in click command")

    def test_component_can_be_hovered(self):
        self.driver.set_dom_element([By.ID, 'an_id'])
        self.driver.set_expected_command(Command.MOVE_TO, {'sessionId': self.driver.session_id,
                                                           'element': self.driver.get_id_for_stored_element(
                                                               [By.ID, 'an_id'])})
        component = UIComponent(self.driver, 'a_component', [By.ID, 'an_id'])
        ##
        component.hover()
        ##
        assert_that(self.driver.has_fulfilled_expectations(), equal_to(True),
                    "clicking on component should result in hoover command")

    def test_has_element(self):
        self.driver.set_dom_element([By.ID, 'an_id'])
        component = UIComponent(self.driver, 'a_component', [By.ID, 'an_id'])
        self.driver.set_dom_element([By.XPATH, './/option'], parent_id=[By.ID, 'dropdownlocator'], children=1,
                                    return_values=[])

        self.driver.set_expected_command(Command.FIND_CHILD_ELEMENTS, {'using': By.ID, 'value': self.driver.
                                         get_id_for_stored_element([By.ID, 'an_id']),
                                         'id': 'id', "sessionId": self.driver.session_id})
        ##
        component.has_element([By.ID, 'another_id'])
        ##
        assert_that(self.driver.has_fulfilled_expectations(), equal_to(True),
                    "component should be able to locate element it is scope")

    def test_can_cache_element(self):
        component = UIComponent(self.driver, 'a_component', [By.ID, 'an_id'])
        component.cache()
        cached_element = MockedWebElement(self.driver, 'another_id')
        ##
        component._cache_web_element(cached_element)
        ##
        assert_that(component.locate(), equal_to(cached_element))


def raise_exception():
    raise NoSuchElementException


class Foo():
    pass



############################################################################
# Copyright 2015 Skyscanner Ltd                                            #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License");          #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#    http://www.apache.org/licenses/LICENSE-2.0                            #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################
from selenium.webdriver import ActionChains
from selenium.webdriver.common.keys import Keys

from pages.ui_component import UIComponent
from pages.wait.wait import Repeat


class TextInput(UIComponent):

    def __init__(self, driver, name, locator=None):
        UIComponent.__init__(self, driver, name, locator)

    def clear(self):
        self.locate().clear()
        return self

    def focus(self):
        self.locate().click()
        Repeat(5, 0.5).until_condition(self._input_focused, "text input is in focus")
        return self

    def input_text(self, text):
        self.locate().send_keys(Keys.CLEAR)
        self.locate().send_keys(text)
        return self

    def input_text_with_keyboard_emulation(self, text):
        """
            Works around the problem of emulating user interactions with text inputs.
            Emulates a key-down action on the first char of the input. This way, implementations which
            require key-down event to trigger auto-suggest are testable.
            Then the chains sends the rest of the text and releases the key.
        """
        ActionChains(self.driver).key_down(text).key_up(Keys.CONTROL).perform()

    def get_placeholder_text(self):
        return self.locate().get_attribute('placeholder')

    def get_current_value(self):
        return self.locate().get_attribute('value')

    def _input_focused(self):
        self.locate().click()
        return self.driver.switch_to.active_element == self.locate()



import unittest

from hamcrest import assert_that, equal_to
from selenium.webdriver.common.by import By
from selenium.webdriver.remote.command import Command

from pages.page import Page
from test.utils.mocks import MockedWebDriver


class ElementWithTraitsTest(unittest.TestCase):
    """
    Unit test for Page class.
    """

    def test_can_load(self):
        driver = MockedWebDriver()

        driver.set_expected_command(Command.GET, {'url': 'http://www.example.com', 'sessionId': driver.session_id})
        ATestPage(driver).load()
        assert_that(driver.has_fulfilled_expectations(), equal_to(True))

    def test_has_element_with_locator(self):
        driver = MockedWebDriver()
        driver.set_expected_command(Command.FIND_ELEMENTS, {"using": By.ID, "sessionId": driver.session_id,
                                                            "value": 'a_locator'})

        ATestPage(driver).has_element_with_locator([By.ID, 'a_locator'])

        assert_that(driver.has_fulfilled_expectations(), equal_to(True), "page should check if it element is present")


class ATestPage(Page):
    def __init__(self, driver):
        super(ATestPage, self).__init__(driver, 'a_test_page')
        self.driver = driver

    def load(self):
        self.driver.get('http://www.example.com')



############################################################################
# Copyright 2015 Skyscanner Ltd                                            #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License");          #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#    http://www.apache.org/licenses/LICENSE-2.0                            #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################
from abc import abstractmethod, ABCMeta
import os

from pages.loadable_element import LoadableElement


DEFAULT_PAGE_TIMEOUT = 30
DEFAULT_PAGE_POLLING = 2


class Page(LoadableElement):
    """
    Abstraction of a generic web page.
    """
    __metaclass__ = ABCMeta

    def __init__(self, driver, name):
        LoadableElement.__init__(self, driver, name)
        self.timeout = int(os.getenv('PAGE_TIMEOUT', DEFAULT_PAGE_TIMEOUT))
        self.polling_time = int(os.getenv('PAGE_POLLING_TIME', DEFAULT_PAGE_POLLING))

    @abstractmethod
    def load(self):
        pass  # pragma: no cover

    def has_element_with_locator(self, locator):
        return len(self.driver.find_elements(locator[0], locator[1])) > 0



############################################################################
# Copyright 2015 Skyscanner Ltd                                            #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License");          #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#    http://www.apache.org/licenses/LICENSE-2.0                            #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################
import unittest
from hamcrest import equal_to, assert_that, raises, calling
from selenium.common.exceptions import NoSuchElementException, TimeoutException

from pages.wait.wait import Wait, Repeat
from test.utils.mocks import MockedWebDriver


class WaitTest(unittest.TestCase):
    """
    Unit test for Wait class.
    """
    def test_set_time_out(self):
        wait = Wait(1).with_timeout(10)
        assert_that(wait._timeout, equal_to(10), 'timeout can be set')

    def test_set_poll_interval(self):
        wait = Wait(10).with_poll_interval(1)
        assert_that(wait._poll, equal_to(1), 'poll interval can be set')

    def test_set_driver(self):
        driver = MockedWebDriver()
        wait = Wait(10).with_driver(driver)
        assert_that(wait._driver.session_id, equal_to(driver.session_id))

    def test_repeat(self):
        assert_that(isinstance(Repeat(1), Wait), equal_to(True))

    def test_ignored_exceptions_can_be_not_iterable(self):
        wait = Wait(1, ignored_exceptions=FirstException)
        assert_that(wait._ignored_exceptions, equal_to((NoSuchElementException, FirstException)))

    def test_ignored_exceptions_can_be_iterable(self):
        wait = Wait(1).with_ignored_exceptions(FirstException, SecondException)
        assert_that(wait._ignored_exceptions, equal_to((NoSuchElementException, FirstException, SecondException)))

    def test_wait_until_gives_an_exceptions_for_a_non_callable(self):
        assert_that(calling(Wait(1).until_condition).with_args(function(), 'a non callable'), raises(TypeError))

    def test_wait_until_returns_evaluated_value(self):
        wait_until = Wait(1).until_condition(function, 'always true')
        assert_that(wait_until, equal_to('the value'))

    def test_wait_until_raises_TimeoutException(self):
        assert_that(calling(Wait(0).until_condition).with_args(always_false, 'always false'), raises(TimeoutException))


class FirstException(Exception):
    pass


class SecondException(Exception):
    pass


def function():
    return 'the value'


def always_false():
    return False



############################################################################
# Copyright 2015 Skyscanner Ltd                                            #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License");          #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#    http://www.apache.org/licenses/LICENSE-2.0                            #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################
import logging

from selenium.common.exceptions import StaleElementReferenceException

from pages.exceptions import IllegalStateException
from pages.traits import Trait, evaluate_traits
from pages.wait.wait import Wait

logger = logging.getLogger()

DEFAULT_POLLING_TIME = 0.5
DEFAULT_TIMEOUT = 25


class ElementWithTraits(object):
    """
    Base class which defines a mechanism to address timing issues on waiting for loading of elements in a tests.
    A trait is a condition that must be verified for the element to be ready.
    """
    def __init__(self, name):
        self.name = name
        self.traits = []
        self.timeout = DEFAULT_TIMEOUT
        self.polling_time = DEFAULT_POLLING_TIME

    def add_trait(self, condition, description):
        self.traits.append(Trait(condition, description))
        return self

    def wait_until_loaded(self, timeout=DEFAULT_TIMEOUT, polling_time=DEFAULT_POLLING_TIME):
        if timeout:
            self.timeout = timeout
        if polling_time:
            self.polling_time = polling_time
        wait = Wait(self.timeout, self.polling_time).with_ignored_exceptions(StaleElementReferenceException)
        if len(self.traits) == 0:
            raise IllegalStateException("Element '{0}' has no traits".format(self.name))
        else:
            wait.until_traits_are_present(self.traits)
            return self

    def has_all_traits(self):
        return len(evaluate_traits(self.traits)) == 0



############################################################################
# Copyright 2015 Skyscanner Ltd                                            #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License");          #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#    http://www.apache.org/licenses/LICENSE-2.0                            #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################
import unittest

from hamcrest import assert_that, equal_to
from selenium.webdriver.common.by import By
from selenium.webdriver.remote.command import Command

from pages.standard_components.image import Image
from test.utils.mocks import MockedWebDriver


class ButtonTest(unittest.TestCase):
    """
    Unit test for Image class.
    """

    def __init__(self, methodName='runTest'):
        super(ButtonTest, self).__init__(methodName)
        self.driver = MockedWebDriver()

    def test_button_executes_click(self):
        self.driver.set_dom_element([By.ID, 'image'], return_values=[{'name': {'src': 'attr'}}])

        self.driver.set_expected_command(Command.GET_ELEMENT_ATTRIBUTE, {'sessionId': self.driver.session_id,
                                         'id': self.driver.get_id_for_stored_element([By.ID, 'image'])})
        #
        image_src = Image(self.driver, 'image', [By.ID, 'image']).get_src()
        #
        assert_that(image_src, equal_to('attr'))
        assert_that(self.driver.has_fulfilled_expectations(), equal_to(True))



############################################################################
# Copyright 2015 Skyscanner Ltd                                            #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License");          #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#    http://www.apache.org/licenses/LICENSE-2.0                            #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################
import unittest

from hamcrest import assert_that, calling, raises, equal_to, ends_with

import pages

from pages.traits import Trait, evaluate_traits


class TraitsTest(unittest.TestCase):
    """
    Unit test for Traits class.
    """
    def test_is_loaded(self):
        assert_that(calling(a_malformed_trait), raises(TypeError))

    def test_str(self):
        assert_that(a_trait().__str__(), ends_with('always present'))

    def test_lazy_evaluation(self):
        traits = get_traits_with_lazy_evaluation(True)

        assert_that(evaluate_traits(traits)[0], equal_to('always true #1'))

    def test_eager_evaluation(self):
        traits = get_traits_with_lazy_evaluation(False)

        assert_that(evaluate_traits(traits)[1], equal_to('always true #2'))


def get_traits_with_lazy_evaluation(value):
        lazy_evaluated_trait = pages.traits
        lazy_evaluated_trait.__dict__['LAZY_EVALUATION'] = value

        first_trait = lazy_evaluated_trait.Trait(lambda: False, 'always true #1')
        second_trait = lazy_evaluated_trait.Trait(lambda: False, 'always true #2')
        return [first_trait, second_trait]


def foo():
    pass


def a_malformed_trait():
    return Trait(foo(), 'a non callable condition')


def a_trait():
    return Trait(lambda: True, 'always present')



############################################################################
# Copyright 2015 Skyscanner Ltd                                            #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License");          #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#    http://www.apache.org/licenses/LICENSE-2.0                            #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################
import unittest

from hamcrest import assert_that, equal_to
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.remote.command import Command

from pages.standard_components.textinput import TextInput
from test.utils.mocks import MockedWebDriver


class TextIputTest(unittest.TestCase):
    """
    Unit test for TextInput class.
    """

    def __init__(self, methodName='runTest'):
        super(TextIputTest, self).__init__(methodName)
        self.driver = MockedWebDriver()

    def test_clear(self):
        self.driver.set_dom_element([By.ID, 'input'])
        self.driver.set_expected_command(Command.CLEAR_ELEMENT, {'sessionId': self.driver.session_id,
                                         'id': self.driver.get_id_for_stored_element([By.ID, 'input'])})
        #
        TextInput(self.driver, 'input', [By.ID, 'input']).clear()
        #
        assert_that(self.driver.has_fulfilled_expectations(), equal_to(True),
                    "clearing text input should fulfill test expectations")

    def test_focus(self):
        self.driver.set_dom_element([By.ID, 'input'])
        self.driver.set_expected_command(Command.CLICK_ELEMENT, {'sessionId': self.driver.session_id,
                                         'id': self.driver.get_id_for_stored_element([By.ID, 'input'])})
        self.driver.set_expected_command(Command.CLICK_ELEMENT, {'sessionId': self.driver.session_id,
                                         'id': self.driver.get_id_for_stored_element([By.ID, 'input'])})
        self.driver.set_expected_command(Command.GET_ACTIVE_ELEMENT, {'sessionId': self.driver.session_id,
                                         'id': self.driver.get_id_for_stored_element([By.ID, 'input'])})
        #
        TextInput(self.driver, 'input', [By.ID, 'input']).focus()
        #
        assert_that(self.driver.has_fulfilled_expectations(), equal_to(True),
                    "focusing on element should fulfill expectations")

    def test_input_text(self):
        self.driver.set_dom_element([By.ID, 'input'])
        self.driver.set_expected_command(Command.SEND_KEYS_TO_ELEMENT, {'sessionId': self.driver.session_id,
                                         'id': self.driver.get_id_for_stored_element([By.ID, 'input']),
                                                                        'value': Keys.CLEAR})
        self.driver.set_expected_command(Command.SEND_KEYS_TO_ELEMENT, {'sessionId': self.driver.session_id,
                                         'id': self.driver.get_id_for_stored_element([By.ID, 'input']),
                                                                        'value': 'text'})
        #
        TextInput(self.driver, 'input', [By.ID, 'input']).input_text('text')
        #
        assert_that(self.driver.has_fulfilled_expectations(), equal_to(True),
                    "inputting text should fulfill expectations")

    def test_get_placeholder_text(self):
        self.driver.set_dom_element([By.ID, 'input'], return_values=[{'name': {'placeholder': 'placeholder text'}}])
        self.driver.set_expected_command(Command.GET_ELEMENT_ATTRIBUTE, {'sessionId': self.driver.session_id,
                                         'id': self.driver.get_id_for_stored_element([By.ID, 'input'])})
        #
        text = TextInput(self.driver, 'input', [By.ID, 'input']).get_placeholder_text()
        #
        assert_that(self.driver.has_fulfilled_expectations(), equal_to(True),
                    "getting placeholder value should fulfill expectations")
        assert_that(text, equal_to('placeholder text'))

    def test_get_current_value(self):
        self.driver.set_dom_element([By.ID, 'input'], return_values=[{'name': {'value': 'current value'}}])
        self.driver.set_expected_command(Command.GET_ELEMENT_ATTRIBUTE, {'sessionId': self.driver.session_id,
                                         'id': self.driver.get_id_for_stored_element([By.ID, 'input'])})
        #
        text = TextInput(self.driver, 'input', [By.ID, 'input']).get_current_value()
        #
        assert_that(self.driver.has_fulfilled_expectations(), equal_to(True),
                    "getting current value should fulfill expectations")
        assert_that(text, equal_to('current value'))

    def test_input_text_with_keyboard_emulation(self):
        self.driver.set_dom_element([By.ID, 'input'])
        self.driver.set_expected_command(Command.SEND_KEYS_TO_ACTIVE_ELEMENT,
                                         {'sessionId': self.driver.session_id,
                                          'id': self.driver.get_id_for_stored_element([By.ID, 'input']),
                                          'value': 'text to send'})
        self.driver.set_expected_command(Command.SEND_KEYS_TO_ACTIVE_ELEMENT,
                                         {'sessionId': self.driver.session_id,
                                          'id': self.driver.get_id_for_stored_element([By.ID, 'input']),
                                          'value': Keys.CONTROL})
        #
        TextInput(self.driver, 'input', [By.ID, 'input']).input_text_with_keyboard_emulation('text to send')
        #
        assert_that(self.driver.has_fulfilled_expectations(), equal_to(True),
                    "getting current value should fulfill expectations")



############################################################################
# Copyright 2015 Skyscanner Ltd                                            #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License");          #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#    http://www.apache.org/licenses/LICENSE-2.0                            #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################
from pages.ui_component import UIComponent


class Dropdown(UIComponent):
    """
    Simplistic implementation of a dropdown. It does not include (for now) wait mechanism for options expansion.
    """
    def __init__(self, driver, name='dropdown', locator=None):
        UIComponent.__init__(self, driver, name, locator)

    def select_option_by_name(self, option_name):
        self.locate().click()
        self.locate().find_element_by_xpath("./option[text()='{name}']".format(name=option_name)).click()

    def select_option_by_value(self, option_value):
        self.locate().click()
        option = self.locate().find_element_by_xpath("./option[@value='{value}']".format(value=option_value))
        option.click()

    def get_option_labels(self):
        return [option.text for option in self.locate().find_elements_by_xpath(".//option")]



############################################################################
# Copyright 2015 Skyscanner Ltd                                            #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License");          #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#    http://www.apache.org/licenses/LICENSE-2.0                            #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################
from selenium.webdriver.common.by import By

from pages.element_with_language import ElementWithLanguage
from pages.ui_component import UIComponent


class Table(UIComponent, ElementWithLanguage):
    """
    Generic model of a table.
    E.g.
    <table class="atable">
        <tbody>
            <tr>...
            ...
            <tr>...
        <\tbody>
    <\table>
    How to use it:
    table = Table(driver, "table", [By.XPATH, ".//tr"], Item, "item", [By.XPATH, "//table[@class='atable']"])
    table.get_items()

    Notice Item must be a subclass of UIComponent.
    """

    def __init__(self, driver, table_name, item_relative_locator, item_class, item_name, table_locator=None):
        UIComponent.__init__(self, driver, table_name, table_locator)
        ElementWithLanguage.__init__(self)
        if not issubclass(item_class, UIComponent):
            raise TypeError("{0} is not subtype of UIComponent".format(item_name))
        self._item_relative_locator = item_relative_locator
        self._item_class = item_class
        self._item_name = item_name

    def get_items(self):
        if self._item_has_language():
            return [self._item_class(self.driver, "{0} #{1}".format(self._item_name, index)).from_web_element(item)
                        .with_language(self.language) for index, item in
                    self._enumerate_table_elements(self.locate())]
        else:
            return [self._item_class(self.driver, "{0} #{1}".format(self._item_name, index)).from_web_element(item) for
                    index, item in self._enumerate_table_elements(self.locate())]

    def _enumerate_table_elements(self, table):
        return enumerate(self._find_by_locator_in_scope(table))

    def _find_by_locator_in_scope(self, scope_element):
        by = self._item_relative_locator[0]
        locator = self._item_relative_locator[1]
        if by == By.ID:
            return scope_element.find_elements_by_id(locator)  # pragma: no cover
        elif by == By.XPATH:
            return scope_element.find_elements_by_xpath(locator)
        elif by == By.CSS_SELECTOR:  # pragma: no cover
            return scope_element.find_elements_by_css_selector(locator)  # pragma: no cover
        elif by == By.CLASS_NAME:  # pragma: no cover
            return scope_element.find_elements_by_class_name(locator)  # pragma: no cover

    def _item_has_language(self):
        return issubclass(self._item_class, ElementWithLanguage)



############################################################################
# Copyright 2015 Skyscanner Ltd                                            #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License");          #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#    http://www.apache.org/licenses/LICENSE-2.0                            #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################
import unittest

from hamcrest import equal_to, assert_that

from pages.element_with_language import ElementWithLanguage


class ElementWithLanguageTest(unittest.TestCase):
    """
    Unit test for ElementWithLanguage class.
    """

    def __init__(self, methodName='runTest'):
        super(ElementWithLanguageTest, self).__init__(methodName)

    def test_component_can_be_created_from_web_element(self):
        ##
        element = ElementWithLanguage().with_language(ITALIAN)
        ##
        assert_that(element.language, equal_to(ITALIAN),
                    "element should store language")


class Language(object):

    def __init__(self, language_id, locale, country_id, enabled):
        self.language_id = language_id
        self.locale = locale
        self.country_id = country_id
        self.enabled = enabled


ITALIAN = Language("IT", "it-it", "IT", True)



############################################################################
# Copyright 2015 Skyscanner Ltd                                            #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License");          #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#    http://www.apache.org/licenses/LICENSE-2.0                            #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################
import unittest

from hamcrest import assert_that, equal_to, calling, raises
from selenium.webdriver.common.by import By
from selenium.webdriver.remote.command import Command

from pages.element_with_language import ElementWithLanguage
from pages.standard_components.table import Table
from pages.ui_component import UIComponent
from test.utils.mocks import MockedWebDriver


class TableTest(unittest.TestCase):
    """
    Unit test for Table class.
    """

    def __init__(self, methodName='runTest'):
        super(TableTest, self).__init__(methodName)
        self.driver = MockedWebDriver()

    def setUp(self):
        self.driver.reset_dom_elements()

    def test_get_items_with_language(self):
        self.driver.set_dom_element([By.ID, 'table'])
        self.driver.set_dom_element([By.XPATH, './/tr'], parent_id=[By.ID, 'table'], children=4,
                                    return_values=[{'text': 'first'}, {'text': 'second'}, {'text': 'third'},
                                                   {'text': 'fourth'}])
        self.driver.set_expected_command(Command.GET_ELEMENT_TEXT, {'sessionId': self.driver.session_id,
                                                                    'id': self.driver.get_id_for_stored_element(
                                                                        [By.XPATH, './/tr'], 1)})
        self.driver.set_expected_command(Command.GET_ELEMENT_TEXT, {'sessionId': self.driver.session_id,
                                                                    'id': self.driver.get_id_for_stored_element(
                                                                        [By.XPATH, './/tr'], 2)})
        self.driver.set_expected_command(Command.GET_ELEMENT_TEXT, {'sessionId': self.driver.session_id,
                                                                    'id': self.driver.get_id_for_stored_element(
                                                                        [By.XPATH, './/tr'], 3)})
        self.driver.set_expected_command(Command.GET_ELEMENT_TEXT, {'sessionId': self.driver.session_id,
                                                                    'id': self.driver.get_id_for_stored_element(
                                                                        [By.XPATH, './/tr'], 4)})

        #
        items = Table(self.driver, 'table', [By.XPATH, './/tr'], ItemWithLanguage, 'item', [By.ID, 'table']).get_items()
        labels = [i.get_text() for i in items]

        assert_that(labels, equal_to(['first', 'second', 'third', 'fourth']),
                    "It should retrieve labels from stored elements")
        assert_that(self.driver.has_fulfilled_expectations(), equal_to(True),
                    "exercising get_items should result in calling Command.GET_ELEMENT_TEXT a number of times.")

    def test_detects_wrong_item_class(self):
        assert_that(calling(Table).with_args(self.driver, 'table', [By.ID, './/tr'], NonUiComponentItem, 'item',
                                             [By.ID, 'table']), raises(TypeError))

    def test_get_items_without_language(self):
        self.driver.set_dom_element([By.ID, 'table'])
        self.driver.set_dom_element([By.XPATH, './/tr'], parent_id=[By.ID, 'table'], children=4,
                                    return_values=[{'text': 'first'}, {'text': 'second'}, {'text': 'third'},
                                                   {'text': 'fourth'}])
        self.driver.set_expected_command(Command.GET_ELEMENT_TEXT, {'sessionId': self.driver.session_id,
                                                                    'id': self.driver.get_id_for_stored_element(
                                                                        [By.XPATH, './/tr'], 1)})
        self.driver.set_expected_command(Command.GET_ELEMENT_TEXT, {'sessionId': self.driver.session_id,
                                                                    'id': self.driver.get_id_for_stored_element(
                                                                        [By.XPATH, './/tr'], 2)})
        self.driver.set_expected_command(Command.GET_ELEMENT_TEXT, {'sessionId': self.driver.session_id,
                                                                    'id': self.driver.get_id_for_stored_element(
                                                                        [By.XPATH, './/tr'], 3)})
        self.driver.set_expected_command(Command.GET_ELEMENT_TEXT, {'sessionId': self.driver.session_id,
                                                                    'id': self.driver.get_id_for_stored_element(
                                                                        [By.XPATH, './/tr'], 4)})
        #
        items = Table(self.driver, 'table', [By.XPATH, './/tr'], Item, 'item', [By.ID, 'table']).get_items()
        #
        labels = [i.get_text() for i in items]

        assert_that(labels, equal_to(['first', 'second', 'third', 'fourth']),
                    "It should retrieve labels from stored elements")
        assert_that(self.driver.has_fulfilled_expectations(), equal_to(True),
                    "exercising get_items should result in calling Command.GET_ELEMENT_TEXT a number of times.")


class Item(UIComponent):
    def __init__(self, driver, name):
        UIComponent.__init__(self, driver, name)

    def get_text(self):
        return self.locate().text


class ItemWithLanguage(Item, ElementWithLanguage):
    def __init__(self, driver, name):
        Item.__init__(self, driver, name)
        ElementWithLanguage.__init__(self)


class NonUiComponentItem(object):
    pass



############################################################################
# Copyright 2015 Skyscanner Ltd                                            #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License");          #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#    http://www.apache.org/licenses/LICENSE-2.0                            #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################
LAZY_EVALUATION = True


class Trait(object):
    """
    A trait is an abstraction of the condition that must be verified for an element to be ready.
    """
    def __init__(self, condition, description):
        """
        :param condition: it is a callable object that must return a boolean.
        :param description: it is a short description of the condition. E.g. 'page has logo', 'table has 10 elements'
        """
        if not hasattr(condition, '__call__'):
            raise TypeError("condition should be callable")
        self.condition = condition
        self.description = description

    def __str__(self):
        return "condition: " + str(self.condition) + ", " + self.description


def evaluate_traits(traits):
    """
    Evaluates traits and returns a list containing the description of traits which are not true.
    Notice that if LAZY_EVALUATION is set to False all traits are evaluated before returning. Use this option
    only for debugging purposes.
    """
    return_value = []
    for trait in traits:
        if not trait.condition():
            if LAZY_EVALUATION:
                return [trait.description]
            else:
                return_value.append(trait.description)
    return return_value



from itertools import imap
from setuptools import setup, find_packages
import pages


def get_requirements():
    reqs_file = "requirements/requirements.txt"
    try:
        with open(reqs_file) as reqs_file:
            reqs = filter(None, imap(lambda line: line.strip(), reqs_file))
            return reqs
    except IOError:
        pass
    return []

setup(
    name='p-ages',
    version=pages.__version__,
    author='valerio morsella',
    author_email='valerio.morsella@skyscanner.net',
    url='https://github.com/Skyscanner/pages',
    license='ASL v.2.0',
    packages=find_packages(exclude=["test*", "samples"]),
    description="pages is a lightweight page object and component Python library for UI tests",
    long_description=open('./README.rst').read(),
    install_requires=get_requirements(),
    test_suite="test",
    classifiers=["Development Status :: 5 - Production/Stable",
                 "Intended Audience :: Developers",
                 "Programming Language :: Python :: 2.7",
                 "Topic :: Software Development :: Libraries :: Python Modules",
                 "Topic :: Software Development :: Testing"]
)



############################################################################
# Copyright 2015 Skyscanner Ltd                                            #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License");          #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#    http://www.apache.org/licenses/LICENSE-2.0                            #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################
import logging
import time
from selenium.common.exceptions import TimeoutException, NoSuchElementException
from pages.traits import evaluate_traits

POLL_FREQUENCY = 0.5
IGNORED_EXCEPTIONS = [NoSuchElementException]  # list of exceptions ignored during calls to the method
LAZY_EVALUATION = True  # Determines if traits should be all evaluated before returning.

logger = logging.getLogger()


class Wait(object):
    """
        Re-write of Selenium WebDriverWait to allow for:
        - better logging and debugging of failing conditions
        - conditions not containing driver as parameter
        - does not need to pass a driver instance
        - handles loading of traits
    """

    def __init__(self, timeout, poll_frequency=POLL_FREQUENCY, ignored_exceptions=None):
        self._timeout = timeout
        self._poll = poll_frequency
        self._driver = None

        # avoid the divide by zero
        if self._poll == 0:
            self._poll = POLL_FREQUENCY  # pragma: no cover
        exceptions = IGNORED_EXCEPTIONS
        if ignored_exceptions is not None:
            try:
                exceptions.extend(iter(ignored_exceptions))
            except TypeError:  # ignored_exceptions is not iterable
                exceptions.append(ignored_exceptions)
        self._ignored_exceptions = tuple(exceptions)

    def until_condition(self, condition, condition_description):
        """
        Waits until conditions is True or returns a non-None value.
        If any of the trait is still not present after timeout, raises a TimeoutException.
        """
        end_time = time.time() + self._timeout
        count = 1
        while True:
            try:
                if not hasattr(condition, '__call__'):
                    raise TypeError("condition is not callable")
                value = condition()
                if type(value) is bool and value is not False:
                    return value
                elif type(value) is not bool and value is not None:
                    return value
                else:
                    logger.info("#" + str(count) + " - wait until " + condition_description)  # pragma: no cover
            except self._ignored_exceptions as ex:
                logger.info("Captured {0} : {1}".format(str(ex.__class__).replace("<type '", "").replace("'>", ""),
                                                        str(ex)))  # pragma: no cover
            time.sleep(self._poll)
            count += 1
            if time.time() > end_time:
                break
        raise TimeoutException(
            msg="condition <" + condition_description + "> was not true after " + str(self._timeout) + " seconds.")

    def until_traits_are_present(self, traits):
        """
        Waits until all traits are present.
        If any of the traits is still not present after timeout, raises a TimeoutException.
        """
        end_time = time.time() + self._timeout
        count = 1
        missing_traits_descriptions = None
        while True:
            missing_traits_descriptions = []
            try:
                missing_traits_descriptions = evaluate_traits(traits)
                if len(missing_traits_descriptions) == 0:
                    return True
                else:
                    logger.debug("#{0} - wait until all traits are present: <{1}>".format(str(count), '> <'.join(
                        missing_traits_descriptions)))
            except self._ignored_exceptions as ex:  # pragma: no cover
                logger.info("Captured {0}: {1}".format(str(ex.__class__).replace("<type '", "").replace("'>", ""),
                                                       str(ex)))  # pragma: no cover
                pass  # pragma: no cover
            time.sleep(self._poll)
            count += 1
            if time.time() > end_time:
                break
        raise TimeoutException(
            msg="conditions " + '<' + '> <'.join(missing_traits_descriptions) + '>' + " not true after " + str(
                self._timeout) + " seconds.")

    def with_timeout(self, timeout):
        """
        Set timeout value.
        """
        self._timeout = timeout
        return self

    def with_poll_interval(self, poll_interval):
        """
        Set poll interval value.
        """
        self._poll = poll_interval
        return self

    def with_ignored_exceptions(self, *ignored_exceptions):
        """
        Set a list of exceptions that should be ignored inside the wait loop.
        """
        for exception in ignored_exceptions:
            self._ignored_exceptions = self._ignored_exceptions + (exception,)
        return self

    def with_driver(self, driver):
        self._driver = driver
        return self


class Repeat(Wait):
    def __init__(self, timeout, poll_frequency=POLL_FREQUENCY, ignored_exceptions=None):
        Wait.__init__(self, timeout, poll_frequency, ignored_exceptions)



############################################################################
# Copyright 2015 Skyscanner Ltd                                            #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License");          #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#    http://www.apache.org/licenses/LICENSE-2.0                            #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################
from abc import ABCMeta, abstractmethod

from pages.element_with_traits import ElementWithTraits


class LoadableElement(ElementWithTraits):
    """
    Abstract class defining a generic loadable element.
    """

    __metaclass__ = ABCMeta

    def __init__(self, driver, name):
        ElementWithTraits.__init__(self, name)
        self.driver = driver

    @abstractmethod
    def load(self):
        """
        Load component through WebDriver API.
        Usage:
            implement action and return an instance of the class so to allow chaining.
            self.__driver__.get("www.example.com")
            return self
        :return: an instance of the class.
        """

    def is_loaded(self):
        return self.has_all_traits()



############################################################################
# Copyright 2015 Skyscanner Ltd                                            #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License");          #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#    http://www.apache.org/licenses/LICENSE-2.0                            #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################
import unittest

from hamcrest import assert_that, equal_to
from selenium.webdriver.common.by import By
from selenium.webdriver.remote.command import Command

from pages.standard_components.dropdown import Dropdown
from test.utils.mocks import MockedWebDriver


class DropdownTest(unittest.TestCase):
    """
    Unit test for Dropdown class.
    """

    def __init__(self, methodName='runTest'):
        super(DropdownTest, self).__init__(methodName)
        self.driver = MockedWebDriver()

    def setUp(self):
        self.driver.reset_dom_elements()

    def test_dropdown_executes_select_option_by_name(self):
        self.driver.set_dom_elements([[By.ID, 'dropdownlocator'], [By.XPATH, "./option[text()='thename']"]])
        self.driver.set_expected_command(Command.CLICK_ELEMENT, {'sessionId': self.driver.session_id, 'id':
                                         self.driver.get_id_for_stored_element([By.ID, 'dropdownlocator'])})
        self.driver.set_expected_command(Command.CLICK_ELEMENT, {'sessionId': self.driver.session_id, 'id':
                                         self.driver.get_id_for_stored_element([By.XPATH,
                                                                                "./option[text()='thename']"])})
        #
        Dropdown(self.driver, 'dropdown', [By.ID, 'dropdownlocator']).select_option_by_name('thename')
        assert_that(self.driver.has_fulfilled_expectations(), equal_to(True),
                    "fetching option labels should result in executing Comman.CLICK and Command.FIND_CHILD_ELEMENTS")

    def test_dropdown_executes_select_option_by_value(self):
        self.driver.set_dom_elements([[By.ID, 'dropdownlocator'], [By.XPATH, "./option[@value='anoption']"]])

        self.driver.set_expected_command(Command.CLICK_ELEMENT, {'sessionId': self.driver.session_id, 'id':
                                         self.driver.get_id_for_stored_element([By.ID, 'dropdownlocator'])})

        self.driver.set_expected_command(Command.CLICK_ELEMENT, {'sessionId': self.driver.session_id, 'id':
                                         self.driver.get_id_for_stored_element([By.XPATH,
                                                                                "./option[@value='anoption']"])})
        #
        Dropdown(self.driver, 'dropdown', [By.ID, 'dropdownlocator']).select_option_by_value('anoption')
        #
        assert_that(self.driver.has_fulfilled_expectations(), equal_to(True),
                    "fetching option labels should result in executing Comman.CLICK and Command.FIND_CHILD_ELEMENTS")

    def test_dropdown_executes_get_options_label(self):
        self.driver.set_dom_element([By.ID, 'dropdownlocator'])
        self.driver.set_dom_element([By.XPATH, './/option'], parent_id=[By.ID, 'dropdownlocator'], children=4,
                                    return_values=[{'text': 'first'}, {'text': 'second'}, {'text': 'third'},
                                                   {'text': 'fourth'}])
        self.driver.set_expected_command(Command.GET_ELEMENT_TEXT, {'sessionId': self.driver.session_id,
                                         'id': self.driver.get_id_for_stored_element([By.XPATH, './/option'], 1)})
        self.driver.set_expected_command(Command.GET_ELEMENT_TEXT, {'sessionId': self.driver.session_id,
                                         'id': self.driver.get_id_for_stored_element([By.XPATH, './/option'], 2)})
        self.driver.set_expected_command(Command.GET_ELEMENT_TEXT, {'sessionId': self.driver.session_id,
                                         'id': self.driver.get_id_for_stored_element([By.XPATH, './/option'], 3)})
        self.driver.set_expected_command(Command.GET_ELEMENT_TEXT, {'sessionId': self.driver.session_id,
                                         'id': self.driver.get_id_for_stored_element([By.XPATH, './/option'], 4)})
        #
        labels = Dropdown(self.driver, 'dropdown', [By.ID, 'dropdownlocator']).get_option_labels()
        #
        assert_that(labels, equal_to(['first', 'second', 'third', 'fourth']),
                    "It should retrieve labels from stored elements")


class CannedElement:

    def __init__(self, locator, parent=False, children=0):
        self.locator = locator
        self.is_parent = parent
        if children and not parent:
            raise ValueError('cannot specify children number if parent is not set')
        self.children = children



############################################################################
# Copyright 2015 Skyscanner Ltd                                            #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License");          #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#    http://www.apache.org/licenses/LICENSE-2.0                            #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.remote.webelement import WebElement

from pages.element_with_traits import ElementWithTraits


class UIComponent(ElementWithTraits):
    """
        Base class representing a generic component in the DOM of a web page. It cannot be instantiated.
        The responsibility of this class is providing the lazy creation of a WebElement.
        This is done on calling the locate() method.

        Usage:
            Subclass, e.g. AnElement(UIComponent), and add traits to it.
            Construction can be done from a locator:
                super(AnElement, self).__init__(a_driver, a_locator)
            or from an already existing WebElement
                super(AnElement, self).__init__(a_driver).from_web_element(an_element)
    """

    def __init__(self, driver, name, locator=None):
        ElementWithTraits.__init__(self, name)
        self.driver = driver
        self.__locator = locator
        self.__web_element = None
        self.__cache = False

    def from_web_element(self, web_element):
        """
            Store reference to a WebElement instance representing the element on the DOM.
            Use it when an instance of WebElement has already been created (e.g. as the result of find_element) and
            you want to create a UIComponent out of it without evaluating it from the locator again.
            Returns an instance of the class.
        """
        if isinstance(web_element, WebElement) is not True:
            raise TypeError("web_element parameter is not of type WebElement.")
        self.__web_element = web_element
        return self

    def cache(self):
        """
            Disclaimer: Dangerous method. Use it only if you know what you are doing.
            Enable caching of the element after lazy evaluation.
            Use in case of elements which are not being updated as results of actions on the page.
            Usage:
                AComponent(a_driver, a_locator).cache()
            Returns an instance of the class.
        """
        self.__cache = True
        return self

    def locate(self):
        """
            Lazily locates the element on the DOM if the WebElement instance is not available already.
            Returns a WebElement object.
            It also caches the element if caching has been set through cache().
        """
        if self.__web_element is not None:
            return self.__web_element
        else:
            locator_type, locator_value = self.__locator
            element = self.driver.find_element(by=locator_type, value=locator_value)
            self._cache_web_element(element)  # cache the element if allowed
            return element

    def is_found(self):
        """
            Evaluates if the element can be found in the DOM.
        """
        try:
            return self.has_all_traits()
        except NoSuchElementException:
            return False

    def is_present(self):
        try:
            self.locate()
            return True
        except NoSuchElementException:
            return False

    def click(self):
        self.locate().click()

    def hover(self):
        ActionChains(self.driver).move_to_element(self.locate()).perform()

    def has_element(self, element_locator):
        """
        Helper method which tries to locate the element within the scope of the current UIComponent.
        :param element_locator: should be in the form of [By.<locator_type>, <locator>]. E.g. [By.ID, "q"]
        """
        return len(self.locate().find_elements(*element_locator)) > 0

    def _cache_web_element(self, element):
        if self.__cache is True:
            self.__web_element = element



############################################################################
# Copyright 2015 Skyscanner Ltd                                            #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License");          #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#    http://www.apache.org/licenses/LICENSE-2.0                            #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################
import unittest

from hamcrest import assert_that, equal_to

from pages.loadable_element import LoadableElement
from test.utils.mocks import MockedWebDriver


class LoadableElementTest(unittest.TestCase):
    """
    Unit test for LoadableElement class.
    """

    def test_is_loaded(self):
        driver = MockedWebDriver()

        element = ALoadableElement(driver).load()
        assert_that(element.is_loaded(), equal_to(True), 'when traits are present element is_loaded() returns True')


class ALoadableElement(LoadableElement):
    def __init__(self, driver):
        super(ALoadableElement, self).__init__(driver, 'loadable_element')
        self.driver = driver
        self.add_trait(lambda: True, 'always present')

    def load(self):
        self.driver.get('http://www.example.com')
        return self
